__all__ = ['factory']

import time

from edge import eventlog
from .exceptions import PsuCtlError
from .types import IUnit

_ACK_TOUT_SEC = 1.0


class _PsuUnit10(IUnit):
    _REF_TO_OPBIT_MAP = {
        ('psu', 'BASE_V1'): 0x0000,
        ('psu', 'BASE_V2'): 0x0001,
        ('psu', 'BASE_V3'): 0x0002,
        ('psu', 'BASE_V4'): 0x0003,
        ('psu', 'BASE_USB1'): 0x000C,
        ('psu', 'BASE_USB2'): 0x000D
    }

    def __init__(self, firmware, ref):
        """
        :type firmware: types.IFirmware
        :type ref: list | tuple(str, str)
        """
        if tuple(ref) not in self._REF_TO_OPBIT_MAP:
            eventlog.log_e('fail: ivalid ref <{}>'.format(ref))
            raise PsuCtlError

        IUnit.__init__(self, firmware, ref)

    def _write_bit(self, op_bit, op_val):
        """
        :type op_bit: int
        :type op_val: bool
        """
        try:
            answer = self._firmware.write_rw_bit(op_bit, op_val, _ACK_TOUT_SEC)
            if answer != op_val:
                eventlog.log_w('fail: write_rw_bit (query:{}, reply:{})'.format(op_val, answer))

        except PsuCtlError:
            eventlog.log_e('fail: write_rw_reg')
            raise

    def run(self, conf):
        """
        :param conf: psuctl.conf#/scripts/<name>/conf
        :type conf: dict
        :rtype: None
        """
        action = conf.get('action')
        if action not in ['on', 'off', 'reboot']:
            eventlog.log_e('fail: unsupported action <{}>'.format(action))
            raise PsuCtlError

        op_bit = self._REF_TO_OPBIT_MAP[tuple(self._ref)]
        if action == 'reboot':
            self._write_bit(op_bit, False)
            time.sleep(1)
            self._write_bit(op_bit, True)

        else:
            self._write_bit(op_bit, action == 'on')


class _PsuUnit14(IUnit):
    _REF_TO_INFO_MAP = {
        ('psu', 'BASE_V1'): 0x1000,
        ('psu', 'BASE_V2'): 0x1001,
        ('psu', 'BASE_V3'): 0x1002,
        ('psu', 'BASE_V4'): 0x1003,
        ('psu', 'BASE_USB1'): 0x100C,
        ('psu', 'BASE_USB2'): 0x100D
    }

    _ACTION_MAP = {
        'on': 0xFF00,
        'off': 0x0000,
        'reboot': 0x0002
    }

    def __init__(self, firmware, ref):
        """
        :type firmware: types.IFirmware
        :type ref: list | tuple(str, str)
        """
        if tuple(ref) not in self._REF_TO_INFO_MAP:
            eventlog.log_e('fail: ivalid ref <{}>'.format(ref))
            raise PsuCtlError

        IUnit.__init__(self, firmware, ref)

    def run(self, conf):
        """
        :param conf: psuctl.conf#/scripts/<name>/conf
        :type conf: dict
        :rtype: None
        """

        action = conf.get('action')
        if action not in ['on', 'off', 'reboot']:
            eventlog.log_e('fail: unsupported action <{}>'.format(action))
            raise PsuCtlError

        op_reg = self._REF_TO_INFO_MAP[tuple(self._ref)]
        op_val = self._ACTION_MAP[action]

        try:
            answer = self._firmware.write_rw_reg(op_reg, op_val, _ACK_TOUT_SEC)
            if op_val != answer:
                eventlog.log_w("fail: write_rw_reg (query:{}, reply:{})".format(op_val, answer))
        except PsuCtlError:
            eventlog.log_e('fail: write_rw_reg')
            raise


def factory(firmware, unit_ref=None):
    """
    :type firmware: types.IFirmware
    :type unit_ref: list | tuple(str, str)
    :rtype: IUnit | None
    """
    firmware_ver = firmware.version
    if firmware_ver in ['v1.0', 'v1.1', 'v1.2', 'v1.3']:
        unit_factory = _PsuUnit10

    elif firmware_ver in ['v1.4']:
        unit_factory = _PsuUnit14

    else:
        unit_factory = _PsuUnit14
        eventlog.log_w('fail: unsupported firmware <{}> (using {})'
                       .format(firmware_ver, unit_factory.__name__))

    return unit_factory(firmware, unit_ref)
